@@ -29,9 +29,9 @@ sub more_results { shift->sth->more_results }
 
 sub affected_rows { shift->{affected_rows} }
 
-sub warnings_count { shift->sth->{mysql_warning_count} }
+sub warnings_count { $_[0]->db->mysql->_dbi_attr($_[0]->sth, 'warning_count') }
 
-sub last_insert_id { shift->sth->{mysql_insertid} }
+sub last_insert_id { $_[0]->db->mysql->_dbi_attr($_[0]->sth, 'insertid') }
 
 sub err { shift->sth->err }
 
@@ -56,8 +56,8 @@ sub _expand {
   # Only expand json columns
   my ($idx, $name) = @$self{qw(idx name)};
   unless ($idx) {
-    my $types = $self->sth->{mysql_type};
-    my @idx = grep { $types->[$_] == 245 or $types->[$_] == 252 } 0 .. $#$types;    # 245 = MySQL, 252 = MariaDB
+    my $types = $self->db->mysql->_dbi_attr($self->sth, 'type');
+    my @idx   = grep { $types->[$_] == 245 or $types->[$_] == 252 } 0 .. $#$types;    # 245 = MySQL, 252 = MariaDB
     ($idx, $name) = @$self{qw(idx name)} = (\@idx, [@{$self->columns}[@idx]]);
   }
 
@@ -227,11 +227,13 @@ Handle multiple results.
   my $affected = $results->affected_rows;
 
 Number of affected rows by the query. The number reported is dependant from
-C<mysql_client_found_rows> option in L<Mojo::mysql>. For example
+C<mysql_client_found_rows> or C<mariadb_client_found_rows> option in
+L<Mojo::mysql>. For example
 
   UPDATE $table SET id = 1 WHERE id = 1
 
-would return 1 if C<mysql_client_found_rows> is set, and 0 otherwise.
+would return 1 if C<mysql_client_found_rows> or L<mariadb_client_found_rows> is
+set, and 0 otherwise.
 
 =head2 last_insert_id
 
